% FLASH: Fast landmark aligned spherical harmonic map
%
% If you use this code in your own work, please cite the following paper:
% [1] P. T. Choi, K. C. Lam, and L. M. Lui, 
%     "FLASH: Fast Landmark Aligned Spherical Harmonic Parameterization for Genus-0 Closed Brain Surfaces."
%     SIAM Journal on Imaging Sciences, vol. 8, no. 1, pp. 67-94, 2015.
%
% Copyright (c) 2013-2022, Gary Pui-Tung Choi
% https://math.mit.edu/~ptchoi/

addpath('mfile')
addpath('data')

%% Example 1 (Brains)

load('Brains1.mat');
lambda = 10;

% If you want to plot your own mesh color-coded by mean curvature, 
% please also run the following lines for the textures:
% mean_curv1 = mean_curvature(v1,f1);
% mean_curv2 = mean_curvature(v2,f2);

% The original meshes
plot_mesh_lm(v1,f1,lm1,mean_curv1);
title('Source Mesh')
plot_mesh_lm(v2,f2,lm2,mean_curv2);
title('Target Mesh')

% The landmark aligned spherical harmonic parameterization
[map,s1,s2] = flash(v1,f1,lm1,v2,f2,lm2,lambda);

% The parameterization results
plot_mesh_lm(s1,f1,lm1,mean_curv1); view([15 -10]);
title('Spherical Conformal Parameterization of the Source Mesh')
plot_mesh_lm(s2,f2,lm2,mean_curv2); view([15 -10]); 
title('Spherical Conformal Parameterization of the Target Mesh')
plot_mesh_lm(map,f1,lm1,mean_curv1); view([15 -10]);
title('Landmark Aligned Spherical Harmonic Parameterization of the Source Mesh')

% To visualize the parameterization result without texture, use:
% plot_mesh(map,f1);

% plot the angle distortion histogram
angle_distortion(v1,f1,map);

%% Example 2 (Brains)

load('Brains2.mat');
lambda = 100;

% The original meshes
plot_mesh_lm(v1,f1,lm1,mean_curv1);
title('Source Mesh')
plot_mesh_lm(v2,f2,lm2,mean_curv2);
title('Target Mesh')


% The landmark aligned spherical harmonic parameterization
[map,s1,s2] = flash(v1,f1,lm1,v2,f2,lm2,lambda);

% The parameterization results
plot_mesh_lm(s1,f1,lm1,mean_curv1); view([-80 -25]); 
title('Spherical Conformal Parameterization of the Source Mesh')
plot_mesh_lm(s2,f2,lm2,mean_curv2); view([-80 -25]); 
title('Spherical Conformal Parameterization of the Target Mesh')
plot_mesh_lm(map,f1,lm1,mean_curv1); view([-80 -25]);
title('Landmark Aligned Spherical Harmonic Parameterization of the Source Mesh')

% plot the angle distortion histogram
angle_distortion(v1,f1,map);

%% Example 3 (Bulldog and Lion)

load('Bulldog_and_Lion.mat');
lambda = 30; 

% The original meshes
plot_mesh_lm(v1,f1,lm1,mean_curv1); view([0 90]);
title('Source Mesh')
plot_mesh_lm(v2,f2,lm2,mean_curv2); view([0 90]);
title('Target Mesh')

% The landmark aligned spherical harmonic parameterization
[map,s1,s2] = flash(v1,f1,lm1,v2,f2,lm2,lambda);

% The parameterization results
plot_mesh_lm(s1,f1,lm1,mean_curv1); view([-65 0]);
title('Spherical Conformal Parameterization of the Source Mesh')
plot_mesh_lm(s2,f2,lm2,mean_curv2); view([-65 0]);
title('Spherical Conformal Parameterization of the Target Mesh')
plot_mesh_lm(map,f1,lm1,mean_curv1); view([-65 0]);
title('Landmark Aligned Spherical Harmonic Parameterization of the Source Mesh')

% plot the angle distortion histogram
angle_distortion(v1,f1,map);
